/*
    Copyright (C) 2012, Aaron Lindsay <aaron@aclindsay.com>

    This file is part of Aedrix.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <init.h>
#include <drivers/serial.h>
#include <types.h>

#define PL011_SERIAL_BASE 0x10009000
#define PL011_SERIAL_FLAG_REGISTER 0x18
#define PL011_SERIAL_BUFFER_FULL (1 << 5)

int pl011_putc(char c)
{
	/* Wait until the serial buffer is empty */
	while (*(volatile uint32*)(PL011_SERIAL_BASE + PL011_SERIAL_FLAG_REGISTER) & (PL011_SERIAL_BUFFER_FULL));

	/* When it's empty, put our character at the base */
	*(volatile uint32*)PL011_SERIAL_BASE = c;

	return 0;
}

struct serial_dev pl011_dev = {
	.name = "pl011",
	.putc = &pl011_putc
};

void pl011_init() {
	serial_register_device(&pl011_dev);
}

early_initcall(pl011_init);
